package dbControl;

import java.sql.Types;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.Map;

/**
 * A parameter in the SQLStatment.
 */
public class SQLParameter
{
	private static final String ERROR_INVOKING_GETTER_METHOD = "Failed to invoke a getter method while setting parameter value";
	private static final String ERROR_SET_PARAMETER_VALUE_FAILED = "Failed to set parameter value";

    /**
     * Parameter name
     */
    private String name;

    /**
     * Parameter value
     */
    private Object value;

    /**
     * Constructor
     * @param name of the parameter
     */
    public SQLParameter(String name)
    {
        this.name = name;
    }

    /**
     * Constructor
     * @param name name of parameter
     * @param value value of parameter
     */
    public SQLParameter(String name, Object value)
    {
        this.name  = name;
        try
        {
	        this.setValue(value);
		}
		catch (Exception e)
		{
			//TODO log exception
		}
    }

	/**
	 * @return a copy of this parameter
	 */
    public Object clone()
    {
        return new SQLParameter(name, value);
    }

	/**
	 * Sets the name of this parameter
	 *
	 * @param name name of the parameter
	 */
    public void setName(String name)
    {
		this.name = name;
	}

	/**
	 * Gets the name of this parameter
	 *
	 * @return name of the parameter
	 */
	public String getName()
	{
		return this.name;
	}

	/**
	 * The parameter name can be made up of multiple components.
	 *   e.g. "employee.address.street"
	 * This method returns each component in a String array.
	 *   e.g. {"employee", "address", "street"}
	 *
	 * @return components of the parameter name
	 */
	public String[] getNameComponents()
	{
		return this.name.split("\\.");
	}

	/**
	 * This method returns the first component in the parameter name.
	 *   e.g. It returns "employee" when the name is "employee.address.street"
	 *
	 * @return the first component in the parameter name
	 */
	public String getFirstNameComponent()
	{
		String[] nameComponents = this.getNameComponents();
		return nameComponents[0];
	}

	/**
	 * Returns the value of this parameter.
	 *
	 * @return the value of this parameter
	 */
	public Object getValue()
	{
		return this.value;
	}

	/**
	 * Sets the value of this parameter
	 *
	 * @param aValue value of this parameter
	 */
    public void setValue(Object aValue) throws Exception
    {
		String[] nameComponents = this.getNameComponents();
		this.value = aValue;
		for (int i = 1; i < nameComponents.length; i++)
		{
			this.value = extractValue(this.value, nameComponents[i]);
		}
	}

	/**
	 * Returns the value of a field in object where the name of the field matches fieldName.
	 */
	private Object extractValue(Object object, String fieldName) throws Exception
	{
		Class objectClass = object.getClass();
		Object value = null;

		String fieldNameCapped = Character.toUpperCase(fieldName.charAt(0)) + fieldName.substring(1);
		Method getMethod = null;
		Method[] methods = objectClass.getMethods();

		//Check if a is<fieldName>() method is available in object.
		String methodName = "is" + fieldNameCapped;
		for (int i = 0; i < methods.length; i++)
		{
			if (methods[i].getName().equals(methodName) &&
				methods[i].getParameterTypes().length == 0 &&
				(methods[i].getReturnType().equals(Boolean.class) ||
				 methods[i].getReturnType().equals(Boolean.TYPE)))
			{
				getMethod = methods[i];
				break;
			}
		}

		if (getMethod == null)
		{
			//Check if a get<fieldName>() method is available in object.
			methodName = "get" + fieldNameCapped;
			for (int i = 0; i < methods.length; i++)
			{
				if (methods[i].getName().equals(methodName) &&
					methods[i].getParameterTypes().length == 0)
				{
					getMethod = methods[i];
					break;
				}
			}
		}

		//If object contains a getter method for fieldName, invoke the method
		//to get the value of fieldName.
		if (getMethod != null)
		{
			try
			{
				value = getMethod.invoke(object, (Object[])null);
			}
			catch (Exception e)
			{
				throw new Exception(ERROR_INVOKING_GETTER_METHOD, e);
			}
			return value;
		}

		//If a field named fieldName exists, return the value of this field.
		Field[] fields = objectClass.getFields();
		for (int i = 0; i < fields.length; i++)
		{
			if (fields[i].getName().equals(fieldName))
			{
				try
				{
					return fields[i].get(object);
				}
				catch (Exception e)
				{
					throw new Exception(ERROR_INVOKING_GETTER_METHOD, e);
				}
			}
		}

		//If object is a Map and it contains a key equals fieldName, then return the value
		//mapped to key.
		if (object instanceof Map && ((Map)object).containsKey(fieldName))
		{
			return ((Map)object).get(fieldName);
		}

		//Failed to get the value of fieldName from object.
		throw new Exception(ERROR_SET_PARAMETER_VALUE_FAILED);
	}

}
