package dbControl;

/**
 * This class parses the statement member of the SQL annotation in the
 * Database Control into a SQLStatement object.
 */
public class SQLParser
{
    //Error messages.
    private static final String ERROR_MISSING_QUOTE = "A quote is missing from SQL statement.";
    private static final String ERROR_MISSING_BRACE = "A closing brace is missing from SQL statement.";
    private static final String ERROR_UNEXPECTED_BRACE = "An unexpected closing brace found in SQL statement.";

    //Points to a position in the SQL statement during parsing.
    private int position = 0;

    //The statement member in a SQL annotation.
    private String statement;

    /**
     * Constructor
     * @param stmt the statement member of a SQL annotation.
     */
    public SQLParser(String stmt)
    {
        this.statement = stmt;
    }

	/**
	 * Specifies the statement member of a SQL annotation to parse.
	 * @param stmt the statment member of a SQL annotation.
	 */
    public void setStatement(String stmt) {
		this.statement = stmt;
	}

    /**
     * Parse a statement member of a SQL annotation into a SQLStatement
     * object
     * @return the SQLStatement representing of statement
     */
    public SQLStatement parse() throws Exception
    {
        SQLStatement sql = new SQLStatement();
        parseStatement(sql);

        if (-1 != position)
            throw new Exception(ERROR_UNEXPECTED_BRACE);

        return sql;
    }


    private void parseEscape(SQLStatement stmt) throws Exception
    {
        int i = findOneOf(statement, " :}", position);

        int closeBrace = statement.indexOf('}', position);

        if (closeBrace < 0)
            throw new Exception(ERROR_MISSING_BRACE);

        String paramName = statement.substring(position, closeBrace);
        this.position = closeBrace;
        stmt.addParameter(new SQLParameter(paramName));

        if (-1 == position || statement.charAt(position) != '}')
            throw new Exception(ERROR_MISSING_BRACE);

        this.position++;
    }

    /**
     * bascially just accept everything as-is, only looking for escapes
     * only need to recognize SQL strings properly
     */
    private void parseStatement(SQLStatement stmt)
            throws Exception
    {
        int index = position;
        while ((index = findOneOf(statement, "{'}", index)) >= 0)
        {
            char ch = statement.charAt(index);

            if (ch == '\'')
            {
                index = statement.indexOf('\'', index + 1);
                if (-1 == index)
                {
                    throw new Exception(ERROR_MISSING_QUOTE);
                }
                index++;
                continue;
            }

            //Append to stmt text up-to, but not including { or }
            stmt.append(statement.substring(position, index));
            position = index;

            if (ch == '}')
                break;

            this.position++;
            parseEscape(stmt);
            index = this.position;
        }

        if (-1 == index)
        {
            stmt.append(statement.substring(position));
            this.position = -1;
        }
    }

    static int findOneOf(String s, String chars, int i)
    {
        if (i == -1)
            return -1;
        for (; i < s.length() ; i++)
            if (-1 != chars.indexOf(s.charAt(i)))
                return i;
        return -1;
    }

}
