package dbControl;

import dbControl.util.PreparedStatementHelper;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Collection;
import java.util.List;

/**
 * This class holds a SQL statement and its parameters.
 */
public class SQLStatement
{
    transient Calendar calendar = null;
    protected CharSequence sql;
    protected List<SQLParameter> parameters;

    /**
     * Default constructor
     */
    public SQLStatement()
    {
        super();
        sql = new StringBuffer();
        parameters = new _ArrayList();
    }

    /**
     * Constructor
     * @param a SQL statement
     */
    public SQLStatement(String sql)
    {
        if (sql != null)
            this.sql = sql;
        else
            this.sql = new StringBuffer();

        this.parameters = new _ArrayList();
    }

    /**
     * Constructor
     * @param a SQL statement
     * @param parameters parameters of the SQL statement
     */
    public SQLStatement(String sql, SQLParameter[] parameters)
    {
        if (sql != null)
            this.sql = sql;
        else
            this.sql = new StringBuffer();

        this.parameters = parameters != null ? Arrays.asList(parameters) : new _ArrayList();
    }

    /**
     * Constructor
     * @param sql SQL statement
     * @param parameters parameters of the SQL statement
     * @param cal calendar for processing dates in the SQL statment.
     */
    public SQLStatement(String sql, SQLParameter[] parameters, Calendar cal)
    {
        if (sql != null)
            this.sql = sql;
        else
            this.sql = new StringBuffer();

        this.parameters = parameters != null ? Arrays.asList(parameters) : new _ArrayList();
        this.calendar = cal;
    }

    /**
     * Returns the SQL statement of this class
     * @return a sql statement
     */
    public String getSQL()
    {
        return sql.toString();
    }

    /**
     * Returns the parameters of the SQL statement in this class
     * @return parameters of the SQL statement in this class
     */
    public SQLParameter[] getParameters()
    {
        if (null == parameters)
            return new SQLParameter[0];
        return (SQLParameter[])parameters.toArray(new SQLParameter[parameters.size()]);
    }

    /**
     * Returns the SQL statement of this class
     * @return a sql statement
     */
    public String toString()
    {
        return sql.toString();
    }

    /**
     * Inserts a string into this SQL statement.
     * @param offset the offset
     * @param str the string
     * @return a reference to this object
     */
    public SQLStatement insert(int offset, String str)
    {
        ((StringBuffer)sql).insert(offset, str);
        return this;
    }

    /**
     * Adds a parameter to this SQL statement
     * @param param a SQL parameter
     * @return a reference to this object.
     */
    protected SQLStatement addParameter(SQLParameter param)
    {
        try { ((StringBuffer)sql).append('?'); } catch (Exception ignore) {}
        parameters.add(param);
        return this;
    }

    /**
     * Adds a collection of paramters to this sql statement
     * @param params a collection of SQL parameters
     */
    protected void addAll(Collection<SQLParameter> params)
    {
        if (null == params)
            return;
        parameters.addAll(params);
    }

    /**
     * Appends a seqence of characters to the SQL statement
     */
    public SQLStatement append(CharSequence sb)
    {
        try { ((StringBuffer)sql).append(sb); } catch (Exception ignore) {}
        return this;
    }

    /**
     * Appends a character to the SQL statement
     */
    public SQLStatement append(char c)
    {
        try { ((StringBuffer)sql).append(c); } catch (Exception ignore) {}
        return this;
    }

    /*
     * Assigns values to the parameters in the SQL parameter by order
     * @param values values to be assigned to the SQL statement's parameters.
     * @throws ControlException

    public void setParameterValuesByPosition(Object[] values) throws Exception{
        for (SQLParameter param : parameters)
        {
            int pos;
            try
            {
                pos = Integer.parseInt(param.name);
                //offset for zero base values array.
                pos -= 1;
            }
            catch (NumberFormatException nfe)
            {
                throw new Exception("Parameter name is not an integer.", nfe);
            }
            if (pos < 0 || pos >= values.length)
            {
                throw new Exception("No argument provided for parameter: " + pos);
            }
            param.value = values[pos];
        }
    }
	*/

    /**
     * This method returns a prepared statement constructed based on the
     * attributes of the class.
     * @ return a prepared statement with its SQL statement and parameters set.
     */
    public PreparedStatement getPreparedStatement(Connection connection) throws SQLException
    {
        PreparedStatement ps = connection.prepareStatement(getSQL());
        for (int i = 0; i < this.parameters.size(); i++)
        {
            //prepared statement index is 1-based not zero-based as the parameters list.
            int pos = i + 1;
            SQLParameter param = this.parameters.get(i);
            PreparedStatementHelper.setPreparedStatementParameter(ps, pos, param.getValue(), calendar);
        }
        return ps;
    }

    private class _ArrayList extends ArrayList
    {
        public void setSize(int size)
        {
            ensureCapacity(size);
            while (size() < size)
                add(null);
        }
        public Object set(int i, Object o)
        {
            if (i >= size())
                setSize(i+1);
            return super.set(i,o);
        }
    }

}
