package dbControl.util;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Map;

public class PreparedStatementHelper
{
    private interface ParameterSetter {

        public static final int NUM_OF_PARAMETER_SETTERS = 5;

        public void setParameter(PreparedStatement ps, int index, Object value, Calendar cal) throws SQLException;

        static ParameterSetter VARCHAR = new ParameterSetter()
        {
            public void setParameter(PreparedStatement ps, int index, Object value, Calendar cal) throws SQLException
            {
                String setValue = value instanceof String ? (String)value : value.toString();
                ps.setObject(index, setValue, Types.VARCHAR);
            }
        };

        static ParameterSetter BOOLEAN = new ParameterSetter()
        {
            public void setParameter(PreparedStatement ps, int index, Object value, Calendar cal) throws SQLException
            {
                if (value instanceof Boolean)
                    ps.setBoolean(index, ((Boolean)value).booleanValue());
            }
        };

        static ParameterSetter TIMESTAMP = new ParameterSetter()
        {
            public void setParameter(PreparedStatement ps, int index, Object value, Calendar cal) throws SQLException
            {
                if (value instanceof java.util.Calendar)
                {
                    Calendar calValue = (Calendar)value;

                    if (null == cal)
                    {
                        /* NOTE: drivers are inconsistent in their handling of setTimestamp(i,date,cal)
                         * so we won't use that, unless the user calls setCalendar().
                         * I'm going with the theory that it makes sense to store
                         * the time relative to the Calendar's timezone rather than
                         * the system timezone otherwise, using a Calendar would be a no-op.
                         */
                        value = new java.sql.Timestamp(
                                calValue.get(Calendar.YEAR) - 1900,
                                calValue.get(Calendar.MONTH),
                                calValue.get(Calendar.DATE),
                                calValue.get(Calendar.HOUR_OF_DAY),
                                calValue.get(Calendar.MINUTE),
                                calValue.get(Calendar.SECOND),
                                calValue.get(Calendar.MILLISECOND));
                    }
                    else
                    {
                        value = new java.sql.Timestamp(calValue.getTimeInMillis());
                    }
                }
                else if (java.util.Date.class.equals(value.getClass()))
                {
                    // some drivers don't like java.util.Date
                    value = new java.sql.Timestamp(((java.util.Date)value).getTime());
                }

                if (value instanceof java.sql.Timestamp)
                {
                    if (null == cal)
                        ps.setTimestamp(index, (java.sql.Timestamp)value);
                    else
                        ps.setTimestamp(index, (java.sql.Timestamp)value, cal);
                    return;
                }
            }
        };

        static ParameterSetter DATE = new ParameterSetter() {
            public void setParameter(PreparedStatement ps, int index, Object value, Calendar cal) throws SQLException
            {
                if (value instanceof java.util.Calendar)
                {
                    /* NOTE: see note above
                     Calendar cal = (Calendar)value;
                     value = new java.sql.Date(cal.getTimeInMillis());
                     ps.setDate(i, (java.sql.Date)value, cal);
                     return;
                     */
                    Calendar calValue = (Calendar)value;

                    if (null == cal)
                    {
                        value = new java.sql.Date(
                                calValue.get(Calendar.YEAR - 1900),
                                calValue.get(Calendar.MONTH),
                                calValue.get(Calendar.DATE));
                    }
                    else
                        value = new java.sql.Date(calValue.getTimeInMillis());
                }
                else if (value.getClass() == java.util.Date.class)
                {
                    // some drivers don't like java.util.Date
                    value = new java.sql.Date(((java.util.Date)value).getTime());
                }

                if (value instanceof java.sql.Date)
                {
                    if (null == cal)
                        ps.setDate(index, (java.sql.Date)value);
                    else
                        ps.setDate(index, (java.sql.Date)value, cal);
                    return;
                }
            }
        };

        static ParameterSetter TIME = new ParameterSetter()
        {
            public void setParameter(PreparedStatement ps, int index, Object value, Calendar cal) throws SQLException
            {
                if (value instanceof java.sql.Time)
                {
                    if (null == cal)
                        ps.setTime(index, (java.sql.Time)value);
                    else
                        ps.setTime(index, (java.sql.Time)value, cal);
                    return;
                }
            }
        };
    }

    private static Map<Integer,ParameterSetter> typeToSetterMap = new HashMap(ParameterSetter.NUM_OF_PARAMETER_SETTERS * 2);

    static
    {
        typeToSetterMap.put(Types.BOOLEAN, ParameterSetter.BOOLEAN);
        typeToSetterMap.put(Types.TIMESTAMP, ParameterSetter.TIMESTAMP);
        typeToSetterMap.put(Types.DATE, ParameterSetter.DATE);
        typeToSetterMap.put(Types.TIME, ParameterSetter.TIME);
        typeToSetterMap.put(Types.VARCHAR, ParameterSetter.VARCHAR);
    }

    public static void setPreparedStatementParameter(PreparedStatement ps, int index, Object value, Calendar calendar)
        throws SQLException
    {
        int sqlType = JavaTypeHelper.getSqlType(value);
        ParameterSetter setter = typeToSetterMap.get(sqlType);
        if (null != setter)
            setter.setParameter(ps, index, value, calendar);
        else if (null == value)
            ps.setNull(index, Types.NULL == sqlType ? Types.VARCHAR : sqlType);
        else
            ps.setObject(index, value, sqlType);

    }
}
