package dbControl.util;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Map;

public class ResultSetHelper
{
    private static final int TYPE_MAX = 29;

    private interface ColumnReader {

        public Object readValue(ResultSet rs, String columnName, GregorianCalendar cal) throws SQLException;

        static ColumnReader UNKNOWN = new ColumnReader()
        {
            public Object readValue(ResultSet rs, String columnName, GregorianCalendar cal) throws SQLException
            {
                return rs.getObject(columnName);
            }
        };

        static ColumnReader BYTE = new ColumnReader()
        {
            public Object readValue(ResultSet rs, String columnName, GregorianCalendar cal) throws SQLException
            {
                return new Byte(rs.getByte(columnName));
            }
        };

        static ColumnReader SHORT = new ColumnReader()
        {
            public Object readValue(ResultSet rs, String columnName, GregorianCalendar cal) throws SQLException
            {
                return new Short(rs.getShort(columnName));
            }
        };

        static ColumnReader INT = new ColumnReader()
        {
            public Object readValue(ResultSet rs, String columnName, GregorianCalendar cal) throws SQLException
            {
                return new Integer(rs.getInt(columnName));
            }
        };

        static ColumnReader LONG = new ColumnReader()
        {
            public Object readValue(ResultSet rs, String columnName, GregorianCalendar cal) throws SQLException
            {
                return new Long(rs.getLong(columnName));
            }
        };

        static ColumnReader FLOAT = new ColumnReader()
        {
            public Object readValue(ResultSet rs, String columnName, GregorianCalendar cal) throws SQLException
            {
                return new Float(rs.getFloat(columnName));
            }
        };

        static ColumnReader DOUBLE = new ColumnReader()
        {
            public Object readValue(ResultSet rs, String columnName, GregorianCalendar cal) throws SQLException
            {
                return new Double(rs.getDouble(columnName));
            }
        };

        static ColumnReader BOOLEAN = new ColumnReader()
        {
            public Object readValue(ResultSet rs, String columnName, GregorianCalendar cal) throws SQLException
            {
                return rs.getBoolean(columnName) ? Boolean.TRUE : Boolean.FALSE;
            }
        };

        static ColumnReader BYTE_OBJ = new ColumnReader()
        {
            public Object readValue(ResultSet rs, String columnName, GregorianCalendar cal) throws SQLException
            {
                byte i = rs.getByte(columnName); return rs.wasNull() ? null : new Byte(i);
            }
        };

        static ColumnReader SHORT_OBJ = new ColumnReader()
        {
            public Object readValue(ResultSet rs, String columnName, GregorianCalendar cal) throws SQLException
            {
                short i = rs.getShort(columnName); return rs.wasNull() ? null : new Short(i);
            }
        };

        static ColumnReader INT_OBJ = new ColumnReader()
        {
            public Object readValue(ResultSet rs, String columnName, GregorianCalendar cal) throws SQLException
            {
                int i = rs.getInt(columnName); return rs.wasNull() ? null : new Integer(i);
            }
        };

        static ColumnReader LONG_OBJ = new ColumnReader()
        {
            public Object readValue(ResultSet rs, String columnName, GregorianCalendar cal) throws SQLException
            {
                long i = rs.getLong(columnName); return rs.wasNull() ? null : new Long(i);
            }
        };

        static ColumnReader FLOAT_OBJ = new ColumnReader()
        {
            public Object readValue(ResultSet rs, String columnName, GregorianCalendar cal) throws SQLException
            {
                float i = rs.getFloat(columnName); return rs.wasNull() ? null : new Float(i);
            }
        };
        static ColumnReader DOUBLE_OBJ = new ColumnReader()
        {
            public Object readValue(ResultSet rs, String columnName, GregorianCalendar cal) throws SQLException
            {
                double i = rs.getDouble(columnName); return rs.wasNull() ? null : new Double(i);
            }
        };

        static ColumnReader BOOLEAN_OBJ = new ColumnReader()
        {
            public Object readValue(ResultSet rs, String columnName, GregorianCalendar cal) throws SQLException
            {
                boolean i = rs.getBoolean(columnName); return rs.wasNull() ? null : (i ? Boolean.TRUE : Boolean.FALSE);
            }
        };

        static ColumnReader BIG_DECIMAL = new ColumnReader()
        {
            public Object readValue(ResultSet rs, String columnName, GregorianCalendar cal) throws SQLException
            {
               return rs.getBigDecimal(columnName);
            }
        };

        static ColumnReader STRING = new ColumnReader()
        {
            public Object readValue(ResultSet rs, String columnName, GregorianCalendar cal) throws SQLException
            {
                return rs.getString(columnName);
            }
        };

        static ColumnReader BYTES = new ColumnReader()
        {
			public Object readValue(ResultSet rs, String columnName, GregorianCalendar cal) throws SQLException
        	{
                return rs.getBytes(columnName);
            }
        };

        static ColumnReader SQLDATE = new ColumnReader()
        {
            public Object readValue(ResultSet rs, String columnName, GregorianCalendar cal) throws SQLException
            {
                if (null == cal)
                    return rs.getDate(columnName);
                else
                    return rs.getDate(columnName, cal);
            }
        };

        static ColumnReader TIME = new ColumnReader()
        {
            public Object readValue(ResultSet rs, String columnName, GregorianCalendar cal) throws SQLException
            {
                if (null == cal)
                    return rs.getTime(columnName);
                else
                    return rs.getTime(columnName, cal);
            }
        };

        static ColumnReader TIMESTAMP = new ColumnReader()
        {
            public Object readValue(ResultSet rs, String columnName, GregorianCalendar cal) throws SQLException
            {
                if (null == cal)
                    return rs.getTimestamp(columnName);
                else
                    return rs.getTimestamp(columnName, cal);
            }
        };

        static ColumnReader STREAM = new ColumnReader()
        {
            public Object readValue(ResultSet rs, String columnName, GregorianCalendar cal) throws SQLException
            {
                //not supported
                return null;
            }
        };

        static ColumnReader READER = new ColumnReader()
        {
            public Object readValue(ResultSet rs, String columnName, GregorianCalendar cal) throws SQLException
            {
                //not supported
                return null;
            }
        };

        static ColumnReader CLOB = new ColumnReader()
        {
            public Object readValue(ResultSet rs, String columnName, GregorianCalendar cal) throws SQLException
            {
                return rs.getClob(columnName);
            }
        };

        static ColumnReader BLOB = new ColumnReader()
        {
            public Object readValue(ResultSet rs, String columnName, GregorianCalendar cal) throws SQLException
            {
                return rs.getBlob(columnName);
            }
        };

        static ColumnReader ARRAY = new ColumnReader()
        {
            public Object readValue(ResultSet rs, String columnName, GregorianCalendar cal) throws SQLException
            {
                return rs.getArray(columnName);
            }
        };

        static ColumnReader REF = new ColumnReader()
        {
            public Object readValue(ResultSet rs, String columnName, GregorianCalendar cal) throws SQLException
            {
                return rs.getRef(columnName);
            }
        };

        static ColumnReader DATE = new ColumnReader()
        {
            public Object readValue(ResultSet rs, String columnName, GregorianCalendar cal) throws SQLException
            {
                java.sql.Timestamp ts = (null == cal) ? rs.getTimestamp(columnName) : rs.getTimestamp(columnName, cal);
                if (null == ts)
                    return null;
                return new java.util.Date(ts.getTime());
            }
        };

        static ColumnReader CALENDAR = new ColumnReader()
        {
            public Object readValue(ResultSet rs, String columnName, GregorianCalendar cal) throws SQLException
            {
                java.sql.Timestamp ts = (null == cal) ? rs.getTimestamp(columnName) : rs.getTimestamp(columnName, cal);
                if (null == ts)
                    return null;
                Calendar c = (null == cal) ? Calendar.getInstance() : (Calendar)cal.clone();
                c.setTimeInMillis(ts.getTime());
                return c;
            }
        };
    }





    private static Map<Class,ColumnReader> typeToReaderMap = new HashMap(TYPE_MAX * 2);

    static
    {
        typeToReaderMap.put(Boolean.TYPE, ColumnReader.BOOLEAN);
        typeToReaderMap.put(Integer.TYPE, ColumnReader.INT);
        typeToReaderMap.put(Long.TYPE, ColumnReader.LONG);
        typeToReaderMap.put(Byte.TYPE, ColumnReader.BYTE);
        typeToReaderMap.put(Short.TYPE, ColumnReader.SHORT);
        typeToReaderMap.put(Float.TYPE, ColumnReader.FLOAT);
        typeToReaderMap.put(Double.TYPE, ColumnReader.DOUBLE);
        typeToReaderMap.put(Boolean.class, ColumnReader.BOOLEAN_OBJ);
        typeToReaderMap.put(Integer.class, ColumnReader.INT_OBJ);
        typeToReaderMap.put(Long.class, ColumnReader.LONG_OBJ);
        typeToReaderMap.put(Byte.class, ColumnReader.BYTE_OBJ);
        typeToReaderMap.put(Short.class, ColumnReader.SHORT_OBJ);
        typeToReaderMap.put(Float.class, ColumnReader.FLOAT_OBJ);
        typeToReaderMap.put(Double.class, ColumnReader.DOUBLE_OBJ);
        typeToReaderMap.put(String.class, ColumnReader.STRING);
        typeToReaderMap.put(java.math.BigDecimal.class, ColumnReader.BIG_DECIMAL);
        typeToReaderMap.put(byte[].class, ColumnReader.BYTES);
        typeToReaderMap.put(java.sql.Timestamp.class, ColumnReader.TIMESTAMP);
        typeToReaderMap.put(java.sql.Time.class, ColumnReader.TIME);
        typeToReaderMap.put(java.sql.Date.class, ColumnReader.SQLDATE);
        typeToReaderMap.put(java.sql.Ref.class, ColumnReader.REF);
        typeToReaderMap.put(java.sql.Blob.class, ColumnReader.BLOB);
        typeToReaderMap.put(java.sql.Clob.class, ColumnReader.CLOB);
        typeToReaderMap.put(java.sql.Array.class, ColumnReader.ARRAY);
        typeToReaderMap.put(java.io.Reader.class, ColumnReader.READER);
        typeToReaderMap.put(java.io.InputStream.class, ColumnReader.STREAM);
        typeToReaderMap.put(java.util.Date.class, ColumnReader.DATE);
        typeToReaderMap.put(java.util.Calendar.class, ColumnReader.CALENDAR);
        typeToReaderMap.put(java.util.GregorianCalendar.class, ColumnReader.CALENDAR);
    }

    public static Object readValue(ResultSet rs, String columnName, Class resultType) throws SQLException
    {
        return readValue(rs, columnName, resultType, null);
    }

    public static Object readValue(ResultSet rs, String columnName, Class resultType, GregorianCalendar cal) throws SQLException
    {
        ColumnReader reader = typeToReaderMap.get(resultType);
        if (reader == null)
            reader = ColumnReader.UNKNOWN;
        return reader.readValue(rs, columnName, cal);
    }
}
